//+------------------------------------------------------------------+ 
//|                                               ColorXOSMA_HTF.mq5 | 
//|                               Copyright  2011, Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+ 
/*
 *         
 * SmoothAlgorithms.mqh  
 *   (): MetaTrader\\MQL5\Include
 * 
 * ColorXOSMA.mq5
 *   (): MetaTrader\\MQL5\Indicators
 */
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link "farria@mail.redcom.ru" 
//----   
#property version   "1.60"
//----     
#property indicator_separate_window 
//----    4
#property indicator_buffers 2 
//----     
#property indicator_plots   1
//+-----------------------------------+
//|       |
//+-----------------------------------+
//----      
#property indicator_type1 DRAW_COLOR_HISTOGRAM
//----      
#property indicator_color1 Gray,OliveDrab,DodgerBlue,DeepPink,Magenta
//----   - 
#property indicator_style1 STYLE_SOLID
//----     5
#property indicator_width1 5
//----   
#property indicator_label1 "XOSMA HTF"
//+-----------------------------------+
//|                 |
//+-----------------------------------+
#define RESET 0 //        
//+-----------------------------------+
//|          |
//+-----------------------------------+
#include <SmoothAlgorithms.mqh> 
//+-----------------------------------+
//|             |
//+-----------------------------------+
enum Applied_price_ // 
  {
   PRICE_CLOSE_ = 1,     //Close
   PRICE_OPEN_,          //Open
   PRICE_HIGH_,          //High
   PRICE_LOW_,           //Low
   PRICE_MEDIAN_,        //Median Price (HL/2)
   PRICE_TYPICAL_,       //Typical Price (HLC/3)
   PRICE_WEIGHTED_,      //Weighted Close (HLCC/4)
   PRICE_SIMPL_,         //Simple Price (OC/2)
   PRICE_QUARTER_,       //Quarted Price (HLOC/4) 
   PRICE_TRENDFOLLOW0_,  //TrendFollow_1 Price 
   PRICE_TRENDFOLLOW1_   //TrendFollow_2 Price 
  };
//+-----------------------------------+
//|         |
//+-----------------------------------+
input ENUM_TIMEFRAMES TimeFrame=PERIOD_H4;      //  
input Smooth_Method XMA_Method=MODE_T3;         //   
input int Fast_XMA = 12;                        //   
input int Slow_XMA = 26;                        //   
input int XPhase=100;                           //   
//--- XPhase:  JJMA    -100 ... +100,     ;
//--- XPhase:  VIDIA   CMO,  AMA    
input Smooth_Method Signal_Method=MODE_JJMA;    //    
input int Signal_XMA=9;                         //    
input int Signal_Phase=100;                     //    (-100..100)
input Applied_price_ AppliedPrice=PRICE_CLOSE_; //  
input int Smooth_XMA=9;                         //    
input int Smooth_Phase=100;                     //   (-100..100)
input bool ReDraw=true;                         //      
//+-----------------------------------+
//----       
bool Init;
//----      
int min_rates_total;
//----      
int XOSMA_Handle;
//----   ,   
//----      
double XOSMABuffer[],ColorXOSMABuffer[];
//+------------------------------------------------------------------+
//|                                    |
//+------------------------------------------------------------------+
string GetStringTimeframe(ENUM_TIMEFRAMES timeframe)
  {
//----
   return(StringSubstr(EnumToString(timeframe),7,-1));
//----
  }
//+------------------------------------------------------------------+    
//| XOSMA indicator initialization function                          | 
//+------------------------------------------------------------------+  
int OnInit()
  {
//----     
   min_rates_total=3;
   Init=true;
//----     
   if(TimeFrame<Period() && TimeFrame!=PERIOD_CURRENT)
     {
      Print("    ColorXOSMA       ");
      Init=false;
      return(INIT_FAILED);
     }
//----    ColorXOSMA
   XOSMA_Handle=iCustom(Symbol(),TimeFrame,"ColorXOSMA",XMA_Method,Fast_XMA,Slow_XMA,XPhase,
                        Signal_Method,Signal_XMA,Signal_Phase,AppliedPrice,Smooth_XMA,Smooth_Phase);
   if(XOSMA_Handle==INVALID_HANDLE)
     {
      Print("      ColorXOSMA");
      Init=false;
      return(INIT_FAILED);
     }
//----    XOSMABuffer   
   SetIndexBuffer(0,XOSMABuffer,INDICATOR_DATA);
//----      
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,min_rates_total);
//---      DataWindow
   PlotIndexSetString(0,PLOT_LABEL,"XOSMA_HTF");
//----   ,      
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,EMPTY_VALUE);
//----       
   ArraySetAsSeries(XOSMABuffer,true);
//----     ,     
   SetIndexBuffer(1,ColorXOSMABuffer,INDICATOR_COLOR_INDEX);
//----       
   ArraySetAsSeries(ColorXOSMABuffer,true);
//----    CXMA   SmoothAlgorithms.mqh
   CXMA XMA;
   string Smooth1=XMA.GetString_MA_Method(XMA_Method);
   string Smooth2=XMA.GetString_MA_Method(Signal_Method);
   string Smooth3=XMA.GetString_MA_Method(MODE_JJMA);
//----      
   string shortname;
   StringConcatenate(shortname,"XOSMA HTF( ",GetStringTimeframe(TimeFrame),", ",
                     Fast_XMA,", ",Slow_XMA,", ",Signal_XMA,", ",Smooth1,", ",Smooth2,", ",Smooth3," )");
//---           
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);
//---     
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits+1);
//----  
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+  
//| XOSMA iteration function                                         | 
//+------------------------------------------------------------------+  
int OnCalculate(const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//----       
   if(rates_total<min_rates_total || !Init) return(RESET);
   if(BarsCalculated(XOSMA_Handle)<Bars(Symbol(),TimeFrame)) return(prev_calculated);
//----   
   int limit,bar;
//----       
   double XOSMA[2];
   datetime XOSMATime[1];
   static uint LastCountBar;
//----     
//----    limit    
   if(prev_calculated>rates_total || prev_calculated<=0)//      
     {
      limit=rates_total-min_rates_total-1; //      
      LastCountBar=rates_total;
     }
   else limit=int(LastCountBar)+rates_total-prev_calculated; //       
//----         
   ArraySetAsSeries(time,true);
//----    
   for(bar=limit; bar>=0 && !IsStopped(); bar--)
     {
      //----      
      XOSMABuffer[bar]=EMPTY_VALUE;
      ColorXOSMABuffer[bar]=0;
      //----      
      if(CopyTime(Symbol(),TimeFrame,time[bar],1,XOSMATime)<=0) return(RESET);

      if(time[bar]>=XOSMATime[0] && time[bar+1]<XOSMATime[0])
        {
         LastCountBar=bar;

         //----      
         if(CopyBuffer(XOSMA_Handle,0,time[bar],2,XOSMA)<=0) return(RESET);

         //----      
         XOSMABuffer[bar]=XOSMA[1];

         if(XOSMABuffer[bar]>0)
           {
            if(XOSMA[1]>XOSMA[0]) ColorXOSMABuffer[bar]=1;
            if(XOSMA[1]<XOSMA[0]) ColorXOSMABuffer[bar]=2;
           }

         if(XOSMABuffer[bar]<0)
           {
            if(XOSMA[1]<XOSMA[0]) ColorXOSMABuffer[bar]=3;
            if(XOSMA[1]>XOSMA[0]) ColorXOSMABuffer[bar]=4;
           }
        }

      if(ReDraw)
        {
         if(XOSMABuffer[bar+1]!=EMPTY_VALUE && XOSMABuffer[bar]==EMPTY_VALUE)
           {
            XOSMABuffer[bar]=XOSMABuffer[bar+1];
            ColorXOSMABuffer[bar]=ColorXOSMABuffer[bar+1];
           }
        }
     }
//----     
   return(rates_total);
  }
//+------------------------------------------------------------------+
